import { RenovateConfigStage } from './common';
export interface RenovateOptionBase {
    admin?: boolean;
    allowedValues?: string[];
    allowString?: boolean;
    cli?: boolean;
    description: string;
    env?: false | string;
    freeChoice?: boolean;
    mergeable?: boolean;
    autogenerated?: boolean;
    name: string;
    parent?: 'hostRules' | 'packageRules' | 'postUpgradeTasks' | 'regexManagers';
    relatedOptions?: string[];
    releaseStatus?: 'alpha' | 'beta' | 'unpublished';
    stage?: RenovateConfigStage;
}
export interface RenovateArrayOption<T extends string | number | Record<string, unknown> = Record<string, unknown>> extends RenovateOptionBase {
    default?: T[];
    mergeable?: boolean;
    type: 'array';
    subType?: 'string' | 'object' | 'number';
}
export interface RenovateStringArrayOption extends RenovateArrayOption<string> {
    format?: 'regex';
    subType: 'string';
}
export interface RenovateNumberArrayOption extends RenovateArrayOption<number> {
    subType: 'number';
}
export interface RenovateBooleanOption extends RenovateOptionBase {
    default?: boolean;
    type: 'boolean';
}
export interface RenovateIntegerOption extends RenovateOptionBase {
    default?: number;
    type: 'integer';
}
export interface RenovateStringOption extends RenovateOptionBase {
    default?: string;
    format?: 'regex';
    replaceLineReturns?: boolean;
    type: 'string';
}
export interface RenovateObjectOption extends RenovateOptionBase {
    default?: any;
    additionalProperties?: Record<string, unknown> | boolean;
    mergeable?: boolean;
    type: 'object';
}
export declare type RenovateOptions = RenovateStringOption | RenovateNumberArrayOption | RenovateStringArrayOption | RenovateIntegerOption | RenovateBooleanOption | RenovateArrayOption | RenovateObjectOption;
export declare function getOptions(): RenovateOptions[];
