export interface Config {
    datasource?: string;
    depName?: string;
    lookupName?: string;
    registryUrls?: string[];
}
export interface DigestConfig extends Config {
    registryUrl?: string;
}
interface ReleasesConfigBase {
    npmrc?: string;
    registryUrls?: string[];
}
export interface GetReleasesConfig extends ReleasesConfigBase {
    lookupName: string;
    registryUrl?: string;
}
export interface GetPkgReleasesConfig extends ReleasesConfigBase {
    datasource: string;
    depName: string;
    lookupName?: string;
    versioning?: string;
    extractVersion?: string;
    constraints?: Record<string, string>;
}
export declare function isGetPkgReleasesConfig(input: unknown): input is GetPkgReleasesConfig;
export interface Release {
    changelogUrl?: string;
    checksumUrl?: string;
    downloadUrl?: string;
    gitRef?: string;
    isDeprecated?: boolean;
    isStable?: boolean;
    releaseTimestamp?: any;
    version: string;
    newDigest?: string;
    constraints?: Record<string, string[]>;
}
export interface ReleaseResult {
    sourceDirectory?: string;
    latestVersion?: string;
    changelogUrl?: string;
    dependencyUrl?: string;
    deprecationMessage?: string;
    display?: string;
    dockerRegistry?: string;
    dockerRepository?: string;
    group?: string;
    homepage?: string;
    name?: string;
    pkgName?: string;
    releases: Release[];
    sourceUrl?: string;
    tags?: Record<string, string>;
    versions?: any;
    registryUrl?: string;
    isPrivate?: boolean;
}
export interface DatasourceApi {
    id: string;
    getDigest?(config: DigestConfig, newValue?: string): Promise<string | null>;
    getReleases(config: GetReleasesConfig): Promise<ReleaseResult | null>;
    defaultRegistryUrls?: string[];
    defaultVersioning?: string;
    defaultConfig?: Record<string, unknown>;
    registryStrategy?: 'first' | 'hunt' | 'merge';
    caching?: boolean;
}
export declare type Datasource = DatasourceApi;
export {};
